@@ -189,7 +189,6 @@ int			xmlIsPubidChar	(int c);
 int			xmlIsLetter	(int c);
 int			xmlIsDigit	(int c);
 int			xmlIsIdeographic(int c);
-int			xmlIsCombining	(int c);
 int			xmlIsExtender	(int c);
 int			xmlIsCombining	(int c);
 int			xmlIsChar	(int c);
@@ -208,7 +207,6 @@ int			xmlSwitchEncoding	(xmlParserCtxtPtr ctxt,
 						 xmlCharEncoding enc);
 int			xmlSwitchToEncoding	(xmlParserCtxtPtr ctxt,
 					     xmlCharEncodingHandlerPtr handler);
-void			xmlFreeParserCtxt	(xmlParserCtxtPtr ctxt);
 
 /**
  * Entities
@@ -275,14 +273,16 @@ int			xmlParseAttributeType	(xmlParserCtxtPtr ctxt,
 						 xmlEnumerationPtr *tree);
 void			xmlParseAttributeListDecl(xmlParserCtxtPtr ctxt);
 xmlElementContentPtr	xmlParseElementMixedContentDecl
-						(xmlParserCtxtPtr ctxt);
+						(xmlParserCtxtPtr ctxt,
+						 xmlParserInputPtr inputchk);
 #ifdef VMS
 xmlElementContentPtr	xmlParseElementChildrenContentD
 						(xmlParserCtxtPtr ctxt);
 #define xmlParseElementChildrenContentDecl	xmlParseElementChildrenContentD
 #else
 xmlElementContentPtr	xmlParseElementChildrenContentDecl
-						(xmlParserCtxtPtr ctxt);
+						(xmlParserCtxtPtr ctxt,
+						 xmlParserInputPtr inputchk);
 #endif
 int			xmlParseElementContentDecl(xmlParserCtxtPtr ctxt,
 						 xmlChar *name,
@@ -394,11 +394,21 @@ void			xmlParserInputShrink	(xmlParserInputPtr in);
 void			htmlInitAutoClose	(void);
 htmlParserCtxtPtr	htmlCreateFileParserCtxt(const char *filename,
 	                                         const char *encoding);
+#endif
 
 /*
  * Specific function to keep track of entities references
  * and used by the XSLT debugger
  */
+/**
+ * xmlEntityReferenceFunc:
+ * @ent: the entity
+ * @firstNode:  the fist node in the chunk
+ * @lastNode:  the last nod in the chunk
+ *
+ * Callback function used when one need to be able to track back the
+ * provenance of a chunk of nodes inherited from an entity replacement
+ */
 typedef	void	(*xmlEntityReferenceFunc)	(xmlEntityPtr ent,
 						 xmlNodePtr firstNode,
 						 xmlNodePtr lastNode);
@@ -406,7 +416,6 @@ typedef	void	(*xmlEntityReferenceFunc)	(xmlEntityPtr ent,
 void		xmlSetEntityReferenceFunc	(xmlEntityReferenceFunc func);
 
 
-#endif
 #ifdef __cplusplus
 }
 #endif
