@@ -962,56 +962,56 @@ xmlIOHTTPOpen (const char *filename) {
  */
 
 void *
-xmlIOHTTPOpenW( const char * post_uri, int compression ) {
+xmlIOHTTPOpenW(const char *post_uri, int compression)
+{
 
-    xmlIOHTTPWriteCtxtPtr	ctxt = NULL;
+    xmlIOHTTPWriteCtxtPtr ctxt = NULL;
 
-    if ( post_uri == NULL )
-	return ( NULL );
+    if (post_uri == NULL)
+        return (NULL);
 
-    ctxt = xmlMalloc( sizeof( xmlIOHTTPWriteCtxt ) );
-    if ( ctxt == NULL ) {
-	xmlGenericError( xmlGenericErrorContext,
-		"xmlIOHTTPOpenW:  Failed to create output HTTP context.\n" );
-	return ( NULL );
+    ctxt = xmlMalloc(sizeof(xmlIOHTTPWriteCtxt));
+    if (ctxt == NULL) {
+        xmlGenericError(xmlGenericErrorContext,
+		    "xmlIOHTTPOpenW:  Failed to create output HTTP context.\n");
+        return (NULL);
     }
 
-    (void)memset( ctxt, 0, sizeof( xmlIOHTTPWriteCtxt ) );
+    (void) memset(ctxt, 0, sizeof(xmlIOHTTPWriteCtxt));
 
-    ctxt->uri = strdup( post_uri );
-    if ( ctxt->uri == NULL ) {
-	xmlGenericError( xmlGenericErrorContext,
-		"xmlIOHTTPOpenW:  Failed to duplicate destination URI.\n" );
-	xmlFreeHTTPWriteCtxt( ctxt );
-	return ( NULL );
+    ctxt->uri = (char *) xmlStrdup((const xmlChar *)post_uri);
+    if (ctxt->uri == NULL) {
+        xmlGenericError(xmlGenericErrorContext,
+		    "xmlIOHTTPOpenW:  Failed to duplicate destination URI.\n");
+        xmlFreeHTTPWriteCtxt(ctxt);
+        return (NULL);
     }
 
     /*
-    **  Since the document length is required for an HTTP post,
-    **  need to put the document into a buffer.  A memory buffer
-    **  is being used to avoid pushing the data to disk and back.
-    */
+     * **  Since the document length is required for an HTTP post,
+     * **  need to put the document into a buffer.  A memory buffer
+     * **  is being used to avoid pushing the data to disk and back.
+     */
 
 #ifdef HAVE_ZLIB_H
-    if ( ( compression > 0 ) && ( compression <= 9 ) ) {
-	
-	ctxt->compression = compression;
-	ctxt->doc_buff    = xmlCreateZMemBuff( compression );
-    }
-    else
+    if ((compression > 0) && (compression <= 9)) {
+
+        ctxt->compression = compression;
+        ctxt->doc_buff = xmlCreateZMemBuff(compression);
+    } else
 #endif
     {
-	/*  Any character conversions should have been done before this  */
+        /*  Any character conversions should have been done before this  */
 
-	ctxt->doc_buff = xmlAllocOutputBuffer( NULL );
+        ctxt->doc_buff = xmlAllocOutputBuffer(NULL);
     }
 
-    if ( ctxt->doc_buff == NULL ) {
-	xmlFreeHTTPWriteCtxt( ctxt );
-	ctxt = NULL;
+    if (ctxt->doc_buff == NULL) {
+        xmlFreeHTTPWriteCtxt(ctxt);
+        ctxt = NULL;
     }
 
-    return ( ctxt );
+    return (ctxt);
 }
 				
 /**
@@ -1966,7 +1966,7 @@ xmlOutputBufferCreateFd(int fd, xmlCharEncodingHandlerPtr encoder) {
     if (ret != NULL) {
         ret->context = (void *) (long) fd;
 	ret->writecallback = xmlFdWrite;
-	ret->closecallback = xmlFdClose;
+	ret->closecallback = NULL;
     }
 
     return(ret);
@@ -2006,7 +2006,7 @@ xmlParserInputBufferCreateIO(xmlInputReadCallback   ioread,
  * @iowrite:  an I/O write function
  * @ioclose:  an I/O close function
  * @ioctx:  an I/O handler
- * @enc:  the charset encoding if known
+ * @encoder:  the charset encoding if known
  *
  * Create a buffered output for the progressive saving
  * to an I/O handler
@@ -2080,6 +2080,19 @@ xmlParserInputBufferPush(xmlParserInputBufferPtr in,
     return(nbchars);
 }
 
+/**
+ * endOfInput:
+ *
+ * When reading from an Input channel indicated end of file or error
+ * don't reread from it again.
+ */
+static int
+endOfInput (void * context ATTRIBUTE_UNUSED,
+	    char * buffer ATTRIBUTE_UNUSED,
+	    int len ATTRIBUTE_UNUSED) {
+    return(0);
+}
+
 /**
  * xmlParserInputBufferGrow:
  * @in:  a buffered parser input
@@ -2125,6 +2138,8 @@ xmlParserInputBufferGrow(xmlParserInputBufferPtr in, int len) {
      */
     if (in->readcallback != NULL) {
 	res = in->readcallback(in->context, &buffer[0], len);
+	if (res <= 0)
+	    in->readcallback = endOfInput;
     } else {
         xmlGenericError(xmlGenericErrorContext,
 		"xmlParserInputBufferGrow : no input !\n");
@@ -2528,6 +2543,8 @@ xmlDefaultExternalEntityLoader(const char *URL, const char *ID,
 	resource = (xmlChar *) URL;
 
     if (resource == NULL) {
+	if (ID == NULL)
+	    ID = "NULL";
 	if ((ctxt->validate) && (ctxt->sax != NULL) && 
             (ctxt->sax->error != NULL))
 	    ctxt->sax->error(ctxt,
